/*
 *  libpipi       Proper image processing implementation library
 *  Copyright (c) 2004-2008 Sam Hocevar <sam@zoy.org>
 *                All Rights Reserved
 *
 *  $Id$
 *
 *  This library is free software. It comes without any warranty, to
 *  the extent permitted by applicable law. You can redistribute it
 *  and/or modify it under the terms of the Do What The Fuck You Want
 *  To Public License, Version 2, as published by Sam Hocevar. See
 *  http://sam.zoy.org/wtfpl/COPYING for more details.
 */

/*
 * stock.c: stock images
 */

#include "config.h"
#include "common.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "pipi.h"
#include "pipi_internals.h"

pipi_image_t *pipi_load_stock(char const *name)
{
    pipi_image_t *ret;
    pipi_pixels_t *pix;
    float *data;

    /* Generate a Bayer dithering pattern. */
    if(!strncmp(name, "bayer:", 6))
    {
        int i, j, w, h, n;

        w = atoi(name + 6);
        name = strchr(name + 6, 'x');
        if(!name)
            return NULL;
        h = atoi(name + 1);
        if(w <= 0 || h <= 0)
            return NULL;

        for(n = 1; n < w || n < h; n *= 2)
            ;

        ret = pipi_new(w, h);
        pix = pipi_getpixels(ret, PIPI_PIXELS_Y_F);
        data = (float *)pix->pixels;

        for(j = 0; j < h; j++)
            for(i = 0; i < w; i++)
            {
                int k, l, x = 0;

                for(k = 1, l = n * n / 4; k < n; k *= 2, l /= 4)
                {
                    if((i & k) && (j & k))
                        x += l;
                    else if(i & k)
                        x += 3 * l;
                    else if(j & k)
                        x += 2 * l;
                }

                data[j * w + i] = (double)(x + 1) / (n * n + 1);
            }

        return ret;
    }

    /* Generate an error diffusion matrix. */
    if(!strncmp(name, "ediff:", 6))
    {
        float const *ker;
        int w, h;

        if(!strcmp(name + 6, "fs"))
        {
            static float const myker[] =
            {
                   0.,     1.,  7./16,
                3./16,  5./16,  1./16,
            };
            ker = myker; w = 3; h = 2;
        }
        else if(!strcmp(name + 6, "jajuni"))
        {
            static float const myker[] =
            {
                   0.,     0.,     1.,  7./48,  5./48,
                3./48,  5./48,  7./48,  5./48,  3./48,
                1./48,  3./48,  5./48,  3./48,  1./48,
            };
            ker = myker; w = 5; h = 3;
        }
        else if(!strcmp(name + 6, "atkinson"))
        {
            static float const myker[] =
            {
                  0.,    1.,  1./8,  1./8,
                1./8,  1./8,  1./8,    0.,
                  0.,  1./8,    0.,    0.,
            };
            ker = myker; w = 4; h = 3;
        }
        else if(!strcmp(name + 6, "fan"))
        {
            static float const myker[] =
            {
                   0.,     0.,     1.,  7./16,
                1./16,  3./16,  5./16,     0.,
            };
            ker = myker; w = 4; h = 2;
        }
        else if(!strcmp(name + 6, "shiaufan"))
        {
            static float const myker[] =
            {
                  0.,    0.,    1.,  1./2,
                1./8,  1./8,  1./4,    0.,
            };
            ker = myker; w = 4; h = 2;
        }
        else if(!strcmp(name + 6, "shiaufan2"))
        {
            static float const myker[] =
            {
                   0.,     0.,    0.,    1.,  1./2,
                1./16,  1./16,  1./8,  1./4,    0.,
            };
            ker = myker; w = 5; h = 2;
        }
        else if(!strcmp(name + 6, "stucki"))
        {
            static float const myker[] =
            {
                   0.,     0.,     1.,  8./42,  4./42,
                2./42,  4./42,  8./42,  4./42,  2./42,
                1./42,  2./42,  4./42,  2./42,  1./42,
            };
            ker = myker; w = 5; h = 3;
        }
        else if(!strcmp(name + 6, "burkes"))
        {
            static float const myker[] =
            {
                   0.,     0.,     1.,  4./16,  2./16,
                1./16,  2./16,  4./16,  2./16,  1./16,
            };
            ker = myker; w = 5; h = 2;
        }
        else if(!strcmp(name + 6, "sierra"))
        {
            static float const myker[] =
            {
                   0.,     0.,     1.,  5./32,  3./32,
                2./32,  4./32,  5./32,  4./32,  2./32,
                   0.,  2./32,  3./32,  2./32,     0.,
            };
            ker = myker; w = 5; h = 3;
        }
        else if(!strcmp(name + 6, "sierra2"))
        {
            static float const myker[] =
            {
                   0.,     0.,     1.,  4./16,  3./16,
                1./16,  2./16,  3./16,  2./16,  1./16,
            };
            ker = myker; w = 5; h = 2;
        }
        else if(!strcmp(name + 6, "lite"))
        {
            static float const myker[] =
            {
                  0.,    1.,  1./2,
                1./4,  1./4,    0.,
            };
            ker = myker; w = 3; h = 2;
        }
        else
            return NULL;

        ret = pipi_new(w, h);
        pix = pipi_getpixels(ret, PIPI_PIXELS_Y_F);
        memcpy(pix->pixels, ker, w * h * sizeof(float));

        return ret;
    }

    /* Generate a completely random image. */
    if(!strncmp(name, "random:", 7))
    {
        unsigned int ctx = 1;
        int x, y, t, w, h;

        w = atoi(name + 7);
        name = strchr(name + 7, 'x');
        if(!name)
            return NULL;
        h = atoi(name + 1);
        if(w <= 0 || h <= 0)
            return NULL;

        ret = pipi_new(w, h);
        pix = pipi_getpixels(ret, PIPI_PIXELS_RGBA_F);
        data = (float *)pix->pixels;

        for(y = 0; y < h; y++)
            for(x = 0; x < w; x++)
            {
                for(t = 0; t < 3; t++)
                {
                    long hi, lo;

                    hi = ctx / 12773L;
                    lo = ctx % 12773L;
                    ctx = 16807L * lo - 2836L * hi;
                    if(ctx <= 0)
                        ctx += 0x7fffffffL;

                    data[4 * (y * w + x) + t]
                        = (double)((ctx % 65536) / 65535.);
                }
                data[4 * (y * w + x) + 3] = 1.0;
            }

        return ret;
    }

    return NULL;
}

